{% extends "layout.html" %}
{% block content %}

<table id="bill_table" class="table table-striped">
    <thead><tr><th>{{ _("Project") }}</th><th>{{ _("Number of members") }}</th><th>{{ _("Number of bills") }}</th><th>{{_("Newest bill")}}</th><th>{{_("Oldest bill")}}</th></tr></thead>
    <tbody>{% for project in projects|sort(attribute='name') %}
        <tr class="{{ loop.cycle("odd", "even") }}">
        <td>{{ project.name }}</td><td>{{ project.members | count }}</td><td>{{ project.get_bills().count() }}</td>
        {% if project.has_bills() %}
        <td>{{ project.get_bills().all()[0].date }}</td>
        <td>{{ project.get_bills().all()[-1].date }}</td>
        {% else %}
        <td></td>
        <td></td>
        {% endif %}
        </tr>
    {% endfor %}
    </tbody>
</table>
{% endblock %}

