{% extends "layout.html" %}

{% block sidebar %}
<ol>
    <li>{{ _("Create the project") }}</li>
    <li><strong>{{ _("Invite people") }}</strong></li>
    <li><a href="{{ url_for(".list_bills") }}">{{ _("Use it!") }}</a></li>
</ol>
{% endblock %}
{% block content %}
<h2>{{ _("Invite people to join this project") }}</h2>
<p>{{ _("Specify a (comma separated) list of email adresses you want to notify about the
creation of this budget management project and we will send them an email for you.") }}</p>
<p>{{ _("If you prefer, you can") }} <a href="{{ url_for(".list_bills") }}">{{ _("skip this step") }}</a> {{ _("and notify them yourself") }}</p>

{% include "display_errors.html" %}
<form class="invites form-horizontal" method="post" accept-charset="utf-8">
    {{ forms.invites(form) }}
</form>
{% endblock %}
